""" Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

import os
from helpers.logger import Logger
from helpers.cmd_utils import CmdUtils
from framework.base.tasks import Tasks
from constants import REPORT_FILE, STEP_RESULT_FILE, LOG_FILE

TASK_STATUS = {"DONE":"DONE", "FAILED":"FAILED"}
TASK_LOG_STATUS = {"STARTED":"STARTED","COMPLETED":"COMPLETED"}
TEXT_COLOR = {'GREEN':'\033[92m', 'RED':'\033[91m', 'YELLOW':'\033[93m', 'RESET':'\033[0m'}
TEXT_DECORATION = {'BLINK':'\033[5m', 'BOLD':'\033[1m','RESET':'\033[0m'}

class FixAuditdRulesForFortiSOAR(Tasks):
    TASK_STATUS_MSG = "Fix the audit rules present in FortiSOAR"
    
    def __init__(self) -> None:
        super().__init__()
        self.logger = Logger.get_logger(__name__)
        self.cmd_line_utilities = CmdUtils()
        self.store_result_path = STEP_RESULT_FILE.format(self.current_version)
        self.report_path = REPORT_FILE.format(self.current_version)

    @property
    def tags(self) -> str:
        return 'post-upgrade'

    def get_description(self) -> str:
        return ""

    def is_supported(self) -> bool:
        current_version = int(self.current_version.replace('.', ''))
        target_upgrade_version = int(
            self.target_upgrade_version.replace('.', ''))
        return target_upgrade_version >= current_version

    def execute(self):
        self.add_banner_in_log_file(self.TASK_STATUS_MSG,TASK_LOG_STATUS["STARTED"])
        step_result = self.get_step_results('pre-upgrade', 'initialize')
        
        self._fix_audit_rule()
            
        self.add_banner_in_log_file(self.TASK_STATUS_MSG,TASK_LOG_STATUS["COMPLETED"])

    def validate(self) -> bool:
        return True

    def _print_status_msg(self, msg, status):
        reset = TEXT_COLOR["RESET"]
        if status == TASK_STATUS["DONE"]:
            color = TEXT_COLOR["GREEN"]
        else:
            color = TEXT_COLOR["RED"]
        truncated_message = msg[:65] + "..." if len(msg) > 65 else msg
        width = 8
        status = f"{status:^{width}}"
        colored_status = f"{color}{status}{reset}"
        final_msg = "{:<70}{}[{}]".format(truncated_message," ",colored_status)
        print(final_msg)
        
    def add_banner_in_log_file(self, msg:str, status: str) -> None:
        status_msg = " [{:^11}] {} {} ".format(status,":",msg)
        border_length = len(status_msg)
        border = '='*border_length
        new_line_char = "\n" if status==TASK_LOG_STATUS["STARTED"] else "\n\n"
        final_msg = f"{status_msg}{new_line_char}"
        if os.path.exists(LOG_FILE):
            with open(LOG_FILE,'a') as log_file:
                log_file.write(final_msg)

    def remove_duplicates(self, file_path):
        seen_lines = set()
        unique_lines = []
        
        # Read the file and keep track of seen lines
        with open(file_path, 'r') as file:
            for line in file:
                if line not in seen_lines:
                    unique_lines.append(line)
                    seen_lines.add(line)
        
        # Write the unique lines back to the file
        with open(file_path, 'w') as file:
            for line in unique_lines:
                file.write(line)
            
    def _fix_audit_rule(self):
        try:
            # file /usr/lib64/dbus-1/dbus-daemon-launch-helper is not present in FortiSOAR by default, 
            # Hence we need to remove the audit rule added for the same.
            f_audit_cis_rules="/etc/audit/rules.d/audit-cis.rules"
            cmd = f"sed -i 's|-a always,exit -F path=/usr/lib64/dbus-1/dbus-daemon-launch-helper -F perm=x -F auid>=1000 -F auid!=4294967295 -k privileged||g' {f_audit_cis_rules}"
            result = self.cmd_line_utilities.execute_cmd(cmd)
            if result['return_code'] != 0:
                self._print_status_msg(self.TASK_STATUS_MSG, TASK_STATUS["FAILED"])
                return
            # audit rules should contain unique entries, otherwise audit rules are not loaded correctly.
            self.remove_duplicates(f_audit_cis_rules)
            cmd = "augenrules --load"
            result = self.cmd_line_utilities.execute_cmd(cmd)
            if result['return_code'] != 0:
                print(result['std_err'])
                self._print_status_msg(self.TASK_STATUS_MSG, TASK_STATUS["FAILED"])
                return
            self._print_status_msg(self.TASK_STATUS_MSG, TASK_STATUS["DONE"])
            
        except Exception as ex:
            err_msg = "ERROR: {}".format(ex)
            self.logger.exception(err_msg)
            self._print_status_msg(self.TASK_STATUS_MSG, TASK_STATUS["FAILED"])
            print(
                f"Exception occurred while adding fixing the auditd rule for FortiSOAR. Refer logs at '{LOG_FILE}'"
            )