""" Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

import os
from helpers.logger import Logger
from helpers.cmd_utils import CmdUtils
from framework.base.tasks import Tasks
from constants import REPORT_FILE, STEP_RESULT_FILE, LOG_FILE

TASK_STATUS = {"DONE":"DONE", "FAILED":"FAILED"}
TASK_LOG_STATUS = {"STARTED":"STARTED","COMPLETED":"COMPLETED"}
TEXT_COLOR = {'GREEN':'\033[92m', 'RED':'\033[91m', 'YELLOW':'\033[93m', 'RESET':'\033[0m'}
TEXT_DECORATION = {'BLINK':'\033[5m', 'BOLD':'\033[1m','RESET':'\033[0m'}

class UpdateEnvironmentFile(Tasks):
    TASK_STATUS_MSG = "Update '/etc/environment' file for offline yum repo"
    
    def __init__(self) -> None:
        super().__init__()
        self.logger = Logger.get_logger(__name__)
        self.cmd_line_utilities = CmdUtils()
        self.store_result_path = STEP_RESULT_FILE.format(self.current_version)
        self.report_path = REPORT_FILE.format(self.current_version)

    @property
    def tags(self) -> str:
        return 'post-upgrade'

    def get_description(self) -> str:
        return ""

    def is_supported(self) -> bool:
        current_version = int(self.current_version.replace('.', ''))
        target_upgrade_version = int(
            self.target_upgrade_version.replace('.', ''))
        return target_upgrade_version >= current_version

    def execute(self):
        self.add_banner_in_log_file(self.TASK_STATUS_MSG,TASK_LOG_STATUS["STARTED"])
        self._update_environment_file()
        self.add_banner_in_log_file(self.TASK_STATUS_MSG,TASK_LOG_STATUS["COMPLETED"])

    def validate(self) -> bool:
        return True

    def _print_status_msg(self, msg, status):
        reset = TEXT_COLOR["RESET"]
        if status == TASK_STATUS["DONE"]:
            color = TEXT_COLOR["GREEN"]
        else:
            color = TEXT_COLOR["RED"]
        truncated_message = msg[:65] + "..." if len(msg) > 65 else msg
        width = 8
        status = f"{status:^{width}}"
        colored_status = f"{color}{status}{reset}"
        final_msg = "{:<70}{}[{}]".format(truncated_message," ",colored_status)
        print(final_msg)
        
    def add_banner_in_log_file(self, msg:str, status: str) -> None:
        status_msg = " [{:^11}] {} {} ".format(status,":",msg)
        border_length = len(status_msg)
        border = '='*border_length
        new_line_char = "\n" if status==TASK_LOG_STATUS["STARTED"] else "\n\n"
        final_msg = f"{status_msg}{new_line_char}"
        if os.path.exists(LOG_FILE):
            with open(LOG_FILE,'a') as log_file:
                log_file.write(final_msg)
        
    def _update_environment_file(self):
        try:
            step_result = self.get_step_results('pre-upgrade', 'initialize')
            i_custom_yum_url = step_result['i_custom_yum_url']
            s_old_yum_repo = step_result['s_old_yum_repo']
            f_etc_environment = step_result['f_etc_environment']

            # The following condition checks for offline repo.
            if i_custom_yum_url == 1:
                if s_old_yum_repo:
                    with open(f_etc_environment, 'r') as env_file:
                        env_data = env_file.read()

                    if s_old_yum_repo in env_data:
                        # Remove the repo.fortisoar.fortinet.com if offline yum repo entry found in the /etc/environment file.
                        cmd = f"sed -i -e \"/{s_old_yum_repo}/d\" -e \"/^$/d\" {f_etc_environment}"
                        result = self.cmd_line_utilities.execute_cmd(cmd)
                        status_code = result['return_code']

                        if status_code != 0:
                            self._print_status_msg(self.TASK_STATUS_MSG, TASK_STATUS["FAILED"])
                            self.print_txt(
                                f"Failed to update the '{f_etc_environment} 'file. Check if the '{f_etc_environment} 'file has write permission.")
                            return
                        self.logger.info("Removed FortiSOAR repo entry as offline yum repo entry found in the '/etc/environment' file")
            self._print_status_msg(self.TASK_STATUS_MSG, TASK_STATUS["DONE"])
        except Exception as ex:
            err_msg = "ERROR: {}".format(ex)
            self.logger.exception(err_msg)
            self._print_status_msg(self.TASK_STATUS_MSG, TASK_STATUS["FAILED"])
            print(
                f"Exception occurred at update environment file task. Refer logs at '{LOG_FILE}'"
            )